@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package YAML::PP;
 
-our $VERSION = '0.004_001'; # TRIAL VERSION
+our $VERSION = '0.004_002'; # TRIAL VERSION
 
 use base 'Exporter';
 our @EXPORT_OK = qw/ Load LoadFile Dump DumpFile /;
@@ -140,12 +140,16 @@ Still TODO:
 
 =item Flow Style
 
-Flow style is not implemented yet, you will get an appropriate error message.
+Flow style is partially implemented.
+
+Not yet working: Implicit flow collection keys, implicit keys in
+flow sequences, content directly after the colon, empty nodes, explicit
+keys
 
 =item Supported Characters
 
-The regexes are not complete. It will not accept characters that should be
-valid, and it will accept characters that should be invalid.
+If you have valid YAML that's not parsed, or the other way round, please
+create an issue.
 
 =item Line Numbers
 
@@ -156,8 +160,6 @@ in some cases.
 
 The error messages need to be improved. Column information is still missing.
 
-=item Unicode
-
 =item Possibly more
 
 =back
