@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package YAML::PP::Lexer;
 
-our $VERSION = '0.004_001'; # TRIAL VERSION
+our $VERSION = '0.004_002'; # TRIAL VERSION
 
 use constant TRACE => $ENV{YAML_PP_TRACE};
 use constant DEBUG => $ENV{YAML_PP_DEBUG} || $ENV{YAML_PP_TRACE};
@@ -25,6 +25,7 @@ sub init {
     $self->{next_line} = [];
     $self->{line} = 0;
     $self->{context} = 'normal';
+    $self->{flowcontext} = 0;
 }
 
 sub next_line { return $_[0]->{next_line} }
@@ -37,6 +38,8 @@ sub set_line { $_[0]->{line} = $_[1] }
 sub inc_line { return $_[0]->{line}++ }
 sub context { return $_[0]->{context} }
 sub set_context { $_[0]->{context} = $_[1] }
+sub flowcontext { return $_[0]->{flowcontext} }
+sub set_flowcontext { $_[0]->{flowcontext} = $_[1] }
 
 my $RE_WS = '[\t ]';
 my $RE_LB = '[\r\n]';
@@ -56,11 +59,15 @@ my $RE_NS_TAG_CHAR = '(?:' . '%[0-9a-fA-F]{2}' .'|'.  q{[0-9A-Za-z#;/?:@&=+$_.*'
 
 #nb-char ::= c-printable - b-char - c-byte-order-mark
 #my $RE_NB_CHAR = '[\x21-\x7E]';
-my $RE_ANCHOR_CAR = '[\x21-\x2B\x2D-\x5A\x5C\x5E-\x7A\x7C\x7E]';
+my $RE_ANCHOR_CAR = '[\x21-\x2B\x2D-\x5A\x5C\x5E-\x7A\x7C\x7E\xA0-\xFF\x{100}-\x{10FFFF}]';
 
 my $RE_PLAIN_START = '[\x21\x22\x24-\x39\x3B-\x7E\xA0-\xFF\x{100}-\x{10FFFF}]';
 my $RE_PLAIN_END = '[\x21-\x39\x3B-\x7E\xA0-\xFF\x{100}-\x{10FFFF}]';
 my $RE_PLAIN_FIRST = '[\x24\x28-\x29\x2B\x2E-\x39\x3B-\x3D\x41-\x5A\x5C\x5E-\x5F\x61-\x7A\x7E\x{100}-\x{10FFFF}]';
+
+my $RE_PLAIN_START_FLOW = '[\x21\x22\x24-\x2B\x2D-\x39\x3B-\x5A\x5C\x5E-\x7A\x7C\x7E\xA0-\xFF\x{100}-\x{10FFFF}]';
+my $RE_PLAIN_END_FLOW = '[\x21-\x2B\x2D-\x39\x3B-\x5A\x5C\x5E-\x7A\x7C\x7E\xA0-\xFF\x{100}-\x{10FFFF}]';
+my $RE_PLAIN_FIRST_FLOW = '[\x24\x28-\x29\x2B\x2E-\x39\x3B-\x3D\x41-\x5A\x5C\x5E-\x5F\x61-\x7A\x7C\x7E\x{100}-\x{10FFFF}]';
 # c-indicators
 #! 21
 #" 22
@@ -88,6 +95,11 @@ my $RE_PLAIN_FIRST_WORD = "(?:[:?-]+$RE_PLAIN_END|$RE_PLAIN_FIRST)(?::+$RE_PLAIN
 my $RE_PLAIN_WORDS = "(?:$RE_PLAIN_FIRST_WORD(?:$RE_WS+$RE_PLAIN_WORD)*)";
 my $RE_PLAIN_WORDS2 = "(?:$RE_PLAIN_WORD(?:$RE_WS+$RE_PLAIN_WORD)*)";
 
+my $RE_PLAIN_WORD_FLOW = "(?::+$RE_PLAIN_END_FLOW|$RE_PLAIN_START_FLOW)(?::+$RE_PLAIN_END_FLOW|$RE_PLAIN_END_FLOW)*";
+my $RE_PLAIN_FIRST_WORD_FLOW = "(?:[:?-]+$RE_PLAIN_END_FLOW|$RE_PLAIN_FIRST_FLOW)(?::+$RE_PLAIN_END_FLOW|$RE_PLAIN_END_FLOW)*";
+my $RE_PLAIN_WORDS_FLOW = "(?:$RE_PLAIN_FIRST_WORD_FLOW(?:$RE_WS+$RE_PLAIN_WORD_FLOW)*)";
+my $RE_PLAIN_WORDS_FLOW2 = "(?:$RE_PLAIN_WORD_FLOW(?:$RE_WS+$RE_PLAIN_WORD_FLOW)*)";
+
 
 #c-secondary-tag-handle  ::= “!” “!”
 #c-named-tag-handle  ::= “!” ns-word-char+ “!”
@@ -122,7 +134,7 @@ my %REGEXES = (
     COLON => qr{(?m:(:)(?=$RE_WS|$))},
     DASH => qr{(?m:(-)(?=$RE_WS|$))},
     DOUBLEQUOTE => qr{(")},
-    DOUBLEQUOTED => qr{(?:\\(?:.|$)|[^"\r\n\\]+)*}m,
+    DOUBLEQUOTED => qr{(?:\\(?:[ \\\/_0abefnrtvLNP"]|x[0-9a-fA-F]{2}|[uU][0-9a-fA-F]+|$)|[^"\r\n\\]+)*}m,
     SINGLEQUOTE => qr{(')},
     SINGLEQUOTED => qr{(?:''|[^'\r\n]+)*},
     LITERAL => qr{(\|)},
@@ -162,109 +174,94 @@ sub _fetch_next_tokens_block_scalar {
     ]);
 }
 
-sub parse_plain_multi {
-    TRACE and warn "=== parse_plain_multi()\n";
-    my ($self, $parser) = @_;
-    my @multi;
-    my $indent = $parser->offset->[ -1 ] + 1;
-    my $tokens = $parser->tokens;
+sub _fetch_next_tokens_plain {
+    my ($self, $column, $indent, $next_line) = @_;
+    my ($spaces, $content) = @$next_line;
+    my $ws = '';
+    if ($content =~ s/\A($RE_WS+)//) {
+        $ws = $1;
+    }
+    if ($content =~ s/\A(#.*)\z//) {
+        $self->push_tokens( [ COMMENT => $spaces . $ws . $1 ]);
+        $self->push_tokens( [ EOL => '' ]);
+        $self->set_context('normal');
+        return;
+    }
 
-    while (1) {
-        my $next_line = $self->fetch_next_line(1) or last;
-        my $lb = $next_line->[2];
-        my $column = 0;
-        my ($spaces, $content) = @$next_line;
-        if ((length $spaces) < $indent) {
-            if (length $content) {
-                last;
-            }
-            push @$tokens, $self->new_token( EOL => $spaces . $lb, column => $column );
-            push @multi, '';
-            @$next_line = ();
-            next;
+    if (not length $content) {
+        $self->push_tokens( [ EOL => $spaces . $ws ] );
+        return;
+    }
+    $self->push_tokens( [ INDENT => $spaces ]);
+
+
+
+        $self->push_tokens( [ WS => $ws ]);
+
+        my $RE = $RE_PLAIN_WORDS2;
+        if ($self->flowcontext) {
+            $RE = $RE_PLAIN_WORDS_FLOW2;
         }
 
-        if ($indent == 0) {
-            last if $content =~ $RE_DOC_END;
-            last if $content =~ $RE_DOC_START;
-        }
-        push @$tokens, $self->new_token( INDENT => $spaces, column => $column );
-        $column += length $spaces;
-        my $ws = '';
-        if ($content =~ s/\A($RE_WS+)//) {
-            $ws = $1;
-        }
-        if ($content =~ s/\A(#.*)\z//) {
-            push @$tokens, $self->new_token( COMMENT => $ws . $1, column => $column );
-            $column += length ($ws . $1);
-            push @$tokens, $self->new_token( EOL => $lb, column => $column );
-            @$next_line = ();
-            last;
-        }
-
-        if (not length $content) {
-            push @$tokens, $self->new_token( EOL => $ws . $lb, column => $column );
-            push @multi, '';
-            next;
-        }
-        push @$tokens, $self->new_token( WS => $ws, column => $column );
-        $column += length $ws;
-        if ($content =~ s/\A($RE_PLAIN_WORDS2)//) {
+        if ($content =~ s/\A($RE)//) {
             my $string = $1;
-            push @$tokens, $self->new_token( PLAIN => $string, column => $column );
-            $column += length $string;
+            $self->push_tokens( [ PLAIN => $string ]);
+            my $ws = '';
             if ($content =~ s/\A($RE_WS+)//) {
-                push @$tokens, $self->new_token( WS => $1, column => $column );
-                $column += length $1;
+                $ws = $1;
             }
-            push @multi, $string;
             if ($content =~ s/\A(#.*)\z//) {
-                push @$tokens, $self->new_token( COMMENT => $1, column => $column );
-                $column += length $1;
-                push @$tokens, $self->new_token( EOL => $lb, column => $column );
-                @$next_line = ();
-                last;
+                $self->push_tokens( [ COMMENT => $ws . $1 ]);
+                $self->push_tokens( [ EOL => '' ]);
+                $self->set_context('normal');
+                return;
             }
             if (length $content) {
-                push @$tokens, $self->new_token( ERROR => $content . $lb, column => $column );
-                @$next_line = ();
-                $self->exception("Unexpected content");
+                $self->push_tokens( [ WS => $ws ]) if $ws;
+                $self->set_context('normal');
+                $next_line->[0] = '';
+                $next_line->[1] = $content;
+                $self->_fetch_next_tokens(1, $indent, $next_line);
+                return;
             }
-            push @$tokens, $self->new_token( EOL => $lb, column => $column );
+            $self->push_tokens( [ EOL => $ws ]);
         }
         else {
-            push @$tokens, $self->new_token( ERROR => $content . $lb, column => $column );
-            @$next_line = ();
-            $self->exception("Unexpected content");
+            if ($self->flowcontext) {
+                $self->set_context('normal');
+                $next_line->[0] = '';
+                $next_line->[1] = $content;
+                $self->_fetch_next_tokens(1, $indent, $next_line);
+                return;
+            }
+            $self->push_tokens( [ ERROR => $content ]);
         }
-    }
-    return \@multi;
 }
 
-
 sub fetch_next_line {
-    my ($self, $force) = @_;
+    my ($self) = @_;
     my $next_line = $self->next_line;
-    if ($force or not defined $next_line->[0] ) {
-        my $line = $self->reader->readline;
-        unless (defined $line) {
-            $self->set_next_line([]);
-            return;
-        }
-        $self->inc_line;
-        $line =~ m/\A( *)([^\r\n]*)([\r\n]|\z)/ or die "Unexpected";
-        @$next_line = ( $1,  $2, $3 );
-        $self->set_next_line($next_line);
+
+    my $line = $self->reader->readline;
+    unless (defined $line) {
+        $self->set_next_line(undef);
+        return;
     }
+    $self->inc_line;
+    $line =~ m/\A( *)([^\r\n]*)([\r\n]|\z)/ or die "Unexpected";
+    $next_line = [ $1,  $2, $3 ];
+    $self->set_next_line($next_line);
+
     return $next_line;
 }
 
 my %fetch_methods = (
     normal => '_fetch_next_tokens',
-    "'" => '_fetch_next_tokens',
-    '"' => '_fetch_next_tokens',
+    "'" => '_fetch_next_tokens_quoted',
+    '"' => '_fetch_next_tokens_quoted',
     block_scalar_start => '_fetch_next_tokens_block_scalar_start',
     block_scalar => '_fetch_next_tokens_block_scalar',
+    plain => '_fetch_next_tokens_plain',
 );
 
 sub fetch_next_tokens {
@@ -272,9 +269,8 @@ sub fetch_next_tokens {
     my $next = $self->next_tokens;
     unless (@$next) {
         my $next_line = $self->fetch_next_line;
-        my $context = $self->context;
         unless ($next_line) {
-            if ($context eq 'block_scalar_start' or $context eq 'block_scalar') {
+            if ($self->context ne 'normal') {
                 $self->push_tokens( [ END => '' ] );
                 $self->set_context('normal');
             }
@@ -282,36 +278,42 @@ sub fetch_next_tokens {
         }
 
         my ($spaces, $content) = @$next_line;
-        my $end;
-        unless ($spaces) {
-            if ($content =~ $RE_DOC_START or $content =~ $RE_DOC_END) {
-                $end = 1;
-            }
-        }
-        if ((length $spaces) < $indent) {
-            if (length $content) {
-                # non-empty less indented line
-                $end = 1;
-            }
-            else {
-                $self->push_tokens( [ EOL => $spaces . $next_line->[2] ] );
-                @$next_line = ();
+        if (not $spaces) {
+            if ($content =~ s/\A(---|\.\.\.)(?=$RE_WS|\z)//) {
+                my $token = $1;
+                if ($self->context ne 'normal') {
+                    $self->push_tokens( [ END => '' ] );
+                    $self->set_context('normal');
+                }
+                my $token_name = { '---' => 'DOC_START', '...' => 'DOC_END' }->{ $token };
+                $self->push_tokens( [ $token_name => $token ] );
+                $next_line->[1] = $content;
+                $self->_fetch_next_tokens(3, $indent, $next_line);
+                if (@$next) {
+                    $next->[-1]->{value} .= $next_line->[2];
+                }
                 return $next;
             }
         }
-        if ($end) {
-            if ($context eq 'block_scalar_start' or $context eq 'block_scalar') {
+        my $context = $self->context;
+        if ((length $spaces) < $indent) {
+            unless (length $content) {
+                $self->push_tokens( [ EOL => $spaces . $next_line->[2] ] );
+                return $next;
+            }
+            # non-empty less indented line
+            if ($context ne 'normal') {
                 $self->push_tokens( [ END => '' ] );
+                $context = 'normal';
+                $self->set_context($context);
             }
-            $context = 'normal';
-            $self->set_context($context);
         }
-        my $method = $fetch_methods{ $self->context };
+        my $method = $fetch_methods{ $context };
+        TRACE and warn __PACKAGE__.':'.__LINE__.": fetch next tokens: $method\n";
         $self->$method(0, $indent, $next_line);
         if (@$next) {
             $next->[-1]->{value} .= $next_line->[2];
         }
-        @$next_line = ();
     }
     return $next;
 }
@@ -327,6 +329,11 @@ my %TOKEN_NAMES = (
     ':' => 'COLON',
     '-' => 'DASH',
     '?' => 'QUESTION',
+    '[' => 'FLOWSEQ_START',
+    ']' => 'FLOWSEQ_END',
+    '{' => 'FLOWMAP_START',
+    '}' => 'FLOWMAP_END',
+    ',' => 'FLOW_COMMA',
 );
 
 my %ANCHOR_ALIAS_TAG =    ( '&' => 1, '*' => 1, '!' => 1 );
@@ -337,8 +344,9 @@ my %FLOW =                ( '{' => 1, '[' => 1, '}' => 1, ']' => 1 );
 
 sub _fetch_next_tokens {
     my ($self, $offset, $indent, $next_line) = @_;
-    my $context = $self->context;
+    my $flowcontext = $self->flowcontext;
     my $next = $self->next_tokens;
+    #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$next_line], ['next_line']);
 
     my $spaces = $next_line->[0];
     my $yaml = \$next_line->[1];
@@ -355,123 +363,27 @@ sub _fetch_next_tokens {
 
     my @tokens;
     if ($offset == 0) {
-        if ($spaces and $context eq 'normal') {
-            if ($$yaml =~ s/\A(#.*\z)//) {
-                push @tokens, ( EOL => $spaces . $1 );
-                $self->push_tokens(\@tokens);
-                return;
-            }
-            if (not length $$yaml) {
-                push @tokens, ( EOL => $spaces );
-                $self->push_tokens(\@tokens);
-                return;
-            }
+        if ($first eq '#') {
+            push @tokens, ( EOL => $spaces . $$yaml );
+            $self->push_tokens(\@tokens);
+            return;
+        }
+        if ($spaces ) {
             push @tokens, ( INDENT => $spaces );
         }
-        elsif (not $spaces and substr($$yaml, 0, 3) eq '---') {
-            if ($$yaml =~ s/$RE_DOC_START//) {
-                push @tokens, ( DOC_START => $1 );
-                if ($$yaml =~ s/\A($RE_EOL|\z)//) {
-                    push @tokens, ( EOL => $1 );
-                    $self->push_tokens(\@tokens);
-                    return;
-                }
-                if ($$yaml =~ s/\A($RE_WS+)//) {
-                    push @tokens, ( WS => $1 );
-                }
-            }
-        }
-        elsif (not $spaces and substr($$yaml, 0, 3) eq '...') {
-            if ($$yaml =~ s/$RE_DOC_END//) {
-                push @tokens, ( DOC_END => $1 );
-                unless ($$yaml =~ s/\A($RE_EOL|\z)//) {
-                    push @tokens, ( 'Expected EOL' => $$yaml );
-                    $self->push_tokens(\@tokens);
-                    return;
-                }
-                push @tokens, ( EOL => $1 );
-                $self->push_tokens(\@tokens);
-                return;
-            }
-        }
-        elsif ($context eq 'normal') {
-            if ($first eq "#") {
-                if ($$yaml =~ s/\A(#.*\z)//) {
-                    push @tokens, ( EOL => $1 );
-                    $self->push_tokens(\@tokens);
-                    return;
-                }
-            }
-            elsif ($first eq "%") {
-                if ($$yaml =~ s/\A(\s*%YAML ?1\.2$RE_WS*)//) {
-                    push @tokens, ( YAML_DIRECTIVE => $1 );
-                }
-                elsif ($$yaml =~ s/\A(\s*%TAG +(!$RE_NS_WORD_CHAR*!|!) +(tag:\S+|!$RE_URI_CHAR+)$RE_WS*)//) {
-                    push @tokens, ( TAG_DIRECTIVE => $1 );
-                    # TODO
-                    my $tag_alias = $2;
-                    my $tag_url = $3;
-                }
-                elsif ($$yaml =~ s/\A(\s*\A%(?:\w+).*)//) {
-                    push @tokens, ( RESERVED_DIRECTIVE => $1 );
-                    warn "Found reserved directive '$1'";
-                }
-                else {
-                    push @tokens, ( 'Invalid directive' => $$yaml );
-                    $self->push_tokens(\@tokens);
-                    return;
-                }
-                if (not length $$yaml) {
-                    push @tokens, ( EOL => '' );
-                }
-                else {
-                    push @tokens, ( 'Invalid directive' => $$yaml );
-                    $self->push_tokens(\@tokens);
-                    return;
-                }
-                $self->push_tokens(\@tokens);
-                return;
-            }
-        }
-        elsif ($QUOTED{ $context }) {
-            my $token_name = $TOKEN_NAMES{ $context };
-            my $token_name2 = $token_name . 'D';
-            my $regex = $REGEXES{ $token_name2 };
-            $token_name2 = $token_name . 'D_LINE';
-
-            my $quoted = $spaces;
-            if ($$yaml =~ s/\A($regex)//) {
-                $quoted .= $1;
-            }
-
-            if ($$yaml =~ s/\A$context//) {
-                push @tokens, ( $token_name2 => $quoted );
-                push @tokens, ( $token_name => $context );
-                $context = 'normal';
-                $self->set_context('normal');
-            }
-            elsif ($$yaml eq '') {
-                $token_name2 = $token_name . 'D_LINE';
-                push @tokens, ( $token_name2 => $quoted );
-                push @tokens, ( EOL => '' );
-                $self->push_tokens(\@tokens);
-                return 1;
-            }
-            else {
-                push @tokens, ( 'Invalid quoted string' => $$yaml );
-                $self->push_tokens(\@tokens);
-                return;
-            }
+        elsif ($first eq "%") {
+            $self->_fetch_next_tokens_directive($yaml);
+            return;
         }
     }
 
-    $first = substr($$yaml, 0, 1);
     while (1) {
         unless (length $$yaml) {
             push @tokens, ( EOL => '' );
             $self->push_tokens(\@tokens);
             return;
         }
+        $first = substr($$yaml, 0, 1);
         my $plain = 0;
 
         if ($QUOTED{ $first }) {
@@ -490,7 +402,7 @@ sub _fetch_next_tokens {
                 push @tokens, ( $token_name2 => $quoted );
                 push @tokens, ( $token_name => $first );
             }
-            elsif ($$yaml eq '') {
+            elsif (not length $$yaml) {
                 push @tokens, ( $token_name . 'D_LINE' => $quoted );
                 push @tokens, ( EOL => '' );
                 $self->push_tokens(\@tokens);
@@ -503,7 +415,6 @@ sub _fetch_next_tokens {
                 return;
             }
 
-
         }
         elsif ($COLON_DASH_QUESTION{ $first }) {
             if ($$yaml =~ s/\A\Q$first\E(?:($RE_WS+)|\z)//) {
@@ -552,7 +463,7 @@ sub _fetch_next_tokens {
                 return;
             }
         }
-        elsif ($first eq ' ') {
+        elsif ($first eq ' ' or $first eq "\t") {
             if ($$yaml =~ s/\A($RE_WS+)//) {
                 my $ws = $1;
                 if ($$yaml =~ s/\A((?:#.*)?\z)//) {
@@ -564,17 +475,34 @@ sub _fetch_next_tokens {
             }
         }
         elsif ($FLOW{ $first }) {
-            push @tokens, ( "Not Implemented: Flow Style" => $$yaml );
-            $self->push_tokens(\@tokens);
-            return;
+            if ($first eq '{' or $first eq '[') {
+                push @tokens, ( $TOKEN_NAMES{ $first } => $first );
+                substr($$yaml, 0, 1, '');
+                $self->set_flowcontext(++$flowcontext);
+            }
+            if ($first eq '}' or $first eq ']') {
+                push @tokens, ( $TOKEN_NAMES{ $first } => $first );
+                substr($$yaml, 0, 1, '');
+                $self->set_flowcontext(--$flowcontext);
+            }
+        }
+        elsif ($first eq ',') {
+            push @tokens, ( $TOKEN_NAMES{ $first } => $first );
+            substr($$yaml, 0, 1, '');
         }
         else {
             $plain = 1;
         }
 
         if ($plain) {
-            if ($$yaml =~ s/\A($RE_PLAIN_WORDS)//) {
-                push @tokens, ( SCALAR => $1 );
+            my $RE = $RE_PLAIN_WORDS;
+#            warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$flowcontext], ['flowcontext']);
+            if ($flowcontext) {
+                $RE = $RE_PLAIN_WORDS_FLOW;
+            }
+            if ($$yaml =~ s/\A($RE)//) {
+                push @tokens, ( PLAIN => $1 );
+#                warn __PACKAGE__.':'.__LINE__.": PLAIN >>$1<<\n";
                 if ($$yaml =~ s/\A(?:($RE_WS+#.*)|($RE_WS*))\z//) {
                     if (defined $1) {
                         push @tokens, ( COMMENT => $1 );
@@ -594,12 +522,92 @@ sub _fetch_next_tokens {
             }
         }
 
-        $first = substr($$yaml, 0, 1);
     }
 
     return;
 }
 
+sub _fetch_next_tokens_quoted {
+    my ($self, $offset, $indent, $next_line) = @_;
+    my $context = $self->context;
+
+    my $spaces = $next_line->[0];
+    my $yaml = \$next_line->[1];
+    if (not length $$yaml) {
+        $self->push_token( EOL => $spaces );
+        return;
+    }
+    # $ESCAPE_CHAR from YAML.pm
+    if ($$yaml =~ tr/\x00-\x08\x0b-\x0c\x0e-\x1f//) {
+        $self->exception("Control characters are not allowed");
+    }
+
+    my @tokens;
+
+    my $token_name = $TOKEN_NAMES{ $context };
+    my $token_name2 = $token_name . 'D';
+    my $regex = $REGEXES{ $token_name2 };
+    $token_name2 = $token_name . 'D_LINE';
+
+    my $quoted = $spaces;
+    if ($$yaml =~ s/\A($regex)//) {
+        $quoted .= $1;
+    }
+
+    if ($$yaml =~ s/\A$context//) {
+        push @tokens, ( $token_name2 => $quoted );
+        push @tokens, ( $token_name => $context );
+        $self->push_tokens(\@tokens);
+        $context = 'normal';
+        $self->set_context('normal');
+        $next_line->[0] = '';
+        $self->_fetch_next_tokens(1, $indent, $next_line);
+    }
+    elsif ($$yaml eq '') {
+        $token_name2 = $token_name . 'D_LINE';
+        push @tokens, ( $token_name2 => $quoted );
+        push @tokens, ( EOL => '' );
+        $self->push_tokens(\@tokens);
+    }
+    else {
+        push @tokens, ( 'Invalid quoted string' => $$yaml );
+        $self->push_tokens(\@tokens);
+    }
+
+}
+
+sub _fetch_next_tokens_directive {
+    my ($self, $yaml) = @_;
+    my @tokens;
+
+    if ($$yaml =~ s/\A(\s*%YAML ?1\.2$RE_WS*)//) {
+        push @tokens, ( YAML_DIRECTIVE => $1 );
+    }
+    elsif ($$yaml =~ s/\A(\s*%TAG +(!$RE_NS_WORD_CHAR*!|!) +(tag:\S+|!$RE_URI_CHAR+)$RE_WS*)//) {
+        push @tokens, ( TAG_DIRECTIVE => $1 );
+        # TODO
+        my $tag_alias = $2;
+        my $tag_url = $3;
+    }
+    elsif ($$yaml =~ s/\A(\s*\A%(?:\w+).*)//) {
+        push @tokens, ( RESERVED_DIRECTIVE => $1 );
+        warn "Found reserved directive '$1'";
+    }
+    else {
+        push @tokens, ( 'Invalid directive' => $$yaml );
+        $self->push_tokens(\@tokens);
+        return;
+    }
+    if (not length $$yaml) {
+        push @tokens, ( EOL => '' );
+    }
+    else {
+        push @tokens, ( 'Invalid directive' => $$yaml );
+    }
+    $self->push_tokens(\@tokens);
+    return;
+}
+
 sub push_token {
     my ($self, $type, $value) = @_;
     my $next = $self->next_tokens;
