@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package YAML::PP::Render;
 
-our $VERSION = '0.004_001'; # TRIAL VERSION
+our $VERSION = '0.004_002'; # TRIAL VERSION
 
 use constant TRACE => $ENV{YAML_PP_TRACE};
 my $WS = '[\t ]';
@@ -37,29 +37,40 @@ sub render_tag {
 }
 
 my %control = (
-    '\\' => '\\', n => "\n", t => "\t", r => "\r", b => "\b",
-    'x0a' => "\n", 'x0d' => "\r",
+    '\\' => '\\', '/' => '/', n => "\n", t => "\t", r => "\r", b => "\b",
+    'a' => "\a", 'b' => "\b", 'e' => "\e", 'f' => "\f", 'v' => "\x0b",
+    'P' => "\x{2029}", L => "\x{2028}", 'N' => "\x85",
+    '0' => "\0", '_' => "\xa0", ' ' => ' ', q/"/ => q/"/,
 );
+
 sub render_quoted {
-    my (%args) = @_;
-    my $double = $args{double};
-    my $lines = $args{lines};
+    my ($self, $info) = @_;
+    my $double = $info->{style} eq '"';
+    my $lines = $info->{value};
+
+    if ($#$lines == 0) {
+        my $quoted = $lines->[0];
+        if ($double) {
+            $quoted =~ s{(?:
+                \\([ \\\/_0abefnrtvLNP"]) | \\x([0-9a-fA-F]{2})
+                | \\u([A-Fa-f0-9]{4}) | \\U([A-Fa-f0-9]{4,8})
+            )}{
+            defined $1 ? $control{ $1 } : defined $2 ? chr hex $2 :
+            defined $3 ? chr hex $3 : chr hex $4
+            }xeg;
+        }
+        else {
+            $quoted =~ s/''/'/g;
+        }
+        $info->{value} = $quoted;
+        return;
+    }
+
     my $quoted = '';
     my $addspace = 0;
+
     for my $i (0 .. $#$lines) {
         my $line = $lines->[ $i ];
-        if ($#$lines == 0) {
-            if ($double) {
-                $line =~ s/\\"/"/g;
-                $line =~ s/\\(x0d|x0a|[\\ntrb])/$control{ $1 }/g;
-                $line =~ s/\\u([A-Fa-f0-9]+)/chr(oct("x$1"))/eg;
-            }
-            else {
-                $line =~ s/''/'/g;
-            }
-            $quoted .= $line;
-            last;
-        }
         my $last = $i == $#$lines;
         my $first = $i == 0;
         if ($line =~ s/^$WS*$/\n/) {
@@ -70,45 +81,42 @@ sub render_quoted {
             else {
                 $quoted .= "\n";
             }
+            next;
         }
-        else {
-            $quoted .= ' ' if $addspace;
-            $addspace = 1;
-            if ($first) {
-            }
-            else {
-                $line =~ s/^$WS+//;
-            }
-            if ($last) {
-            }
-            else {
-                $line =~ s/$WS+$//;
-            }
-            if ($double) {
-                $line =~ s/\\"/"/g;
-            }
-            else {
-                $line =~ s/''/'/g;
-            }
-            if (not $last and $line =~ s/\\$//) {
+
+        $quoted .= ' ' if $addspace;
+        $addspace = 1;
+        if (not $first) {
+            $line =~ s/^$WS+//;
+        }
+        if (not $last) {
+            $line =~ s/$WS+$//;
+        }
+        if ($double) {
+            $line =~ s{(?:
+                \\([ \\\/_0abefnrtvLNP"]) | \\x([0-9a-fA-F]{2})
+                | \\u([A-Fa-f0-9]{4}) | \\U([A-Fa-f0-9]{4,8})
+            )}{
+            defined $1 ? $control{ $1 } : defined $2 ? chr hex $2 :
+            defined $3 ? chr hex $3 : chr hex $4
+            }xeg;
+            if ($line =~ s/\\$//) {
                 $addspace = 0;
             }
-            $line =~ s/^\\ / /;
-            if ($double) {
-                $line =~ s/\\(x0d|x0a|[\\ntrb])/$control{ $1 }/g;
-                $line =~ s/\\u([A-Fa-f0-9]+)/chr(oct("x$1"))/eg;
-            }
-            $quoted .= $line;
         }
+        else {
+            $line =~ s/''/'/g;
+        }
+        $quoted .= $line;
     }
-    return $quoted;
+    $info->{value} = $quoted;
 }
 
 sub render_block_scalar {
-    my (%args) = @_;
-    my $block_type = $args{block_type};
-    my $chomp = $args{chomp} || '';
-    my $lines = $args{lines};
+    my ($self, $info) = @_;
+    my $block_type = $info->{style};
+    my $chomp = $info->{block_chomp} || '';
+    my $lines = $info->{value};
 
     my ($folded, $keep, $trim);
     if ($block_type eq '>') {
@@ -175,11 +183,13 @@ sub render_block_scalar {
         }
     }
     TRACE and warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$string], ['string']);
-    return $string;
+    $info->{value} = $string;
 }
 
 sub render_multi_val {
-    my ($multi) = @_;
+    my ($self, $info) = @_;
+    my $multi = $info->{value};
+    return $multi unless ref $multi;
     # remove empty lines at beginning and end
     while (@$multi and $multi->[0] eq '') {
         shift @$multi;
@@ -205,7 +215,7 @@ sub render_multi_val {
             $start = 0;
         }
     }
-    return $string;
+    $info->{value} = $string;
 }
 
 
